package org.artifactory.traffic;

import org.artifactory.security.access.AccessTokenAuthentication;
import org.artifactory.traffic.v2.TrafficEntryV2;
import org.jfrog.access.common.ServiceId;
import org.jfrog.access.common.ServiceType;
import org.jfrog.access.token.JwtAccessToken;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.UUID;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

public class TrafficUtilsTest {

    @DataProvider(name = "subjects")
    public static Object[][] subjects() {
        return new Object[][] {
                {null, TrafficEntryV2.TrafficActor.USER},
                {"", TrafficEntryV2.TrafficActor.USER},
                {"@", TrafficEntryV2.TrafficActor.USER},
                {"jfxr@", TrafficEntryV2.TrafficActor.USER},
                {"@sded", TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.ACCESS), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.ACCESS_LEGACY), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.ARTIFACTORY), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.ARTIFACTORY_LEGACY), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.MISSION_CONTROL), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.XRAY), TrafficEntryV2.TrafficActor.XRAY},
                {getFormattedName(ServiceType.XRAY_INDEXER), TrafficEntryV2.TrafficActor.XRAY},
                {getFormattedName(ServiceType.XRAY_PERSIST), TrafficEntryV2.TrafficActor.XRAY},
                {getFormattedName(ServiceType.XRAY_ANALYSIS), TrafficEntryV2.TrafficActor.XRAY},
                {getFormattedName(ServiceType.DISTRIBUTION), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.FRONTEND), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.EVENT), TrafficEntryV2.TrafficActor.USER},
                {getFormattedName(ServiceType.PIPELINES), TrafficEntryV2.TrafficActor.PIPELINES},
                {getFormattedName(ServiceType.REPLICATOR), TrafficEntryV2.TrafficActor.REPLICATOR}
        };
    }

    @Test(dataProvider = "subjects")
    public void testAccessToken(String subject, TrafficEntryV2.TrafficActor expected) {
        AccessTokenAuthentication authentication = mock(AccessTokenAuthentication.class);
        JwtAccessToken token = mock(JwtAccessToken.class);
        when(token.getSubject()).thenReturn(subject);
        when(authentication.getAccessToken()).thenReturn(token);
        TrafficEntryV2.TrafficActor actual = TrafficUtils.getTrafficActor(null, null, authentication);
        assertEquals(expected, actual);
    }

    @DataProvider(name = "userdata")
    public static Object[][] userdata() {
        return new Object[][] {
                {"", "", TrafficEntryV2.TrafficActor.USER},
                {null, null, TrafficEntryV2.TrafficActor.USER},
                {"", "xray", TrafficEntryV2.TrafficActor.XRAY},
                {"useragentxray", "", TrafficEntryV2.TrafficActor.XRAY},
                {"useragentjfrog-replicator/", "", TrafficEntryV2.TrafficActor.REPLICATOR},
                {"noise", "", TrafficEntryV2.TrafficActor.USER},
        };
    }

    @Test(dataProvider = "userdata")
    public void testUserData(String userAgent, String username, TrafficEntryV2.TrafficActor expected) {
        TrafficEntryV2.TrafficActor actual = TrafficUtils.getTrafficActor(userAgent, username, null);
        assertEquals(expected, actual);
    }

    private static String getFormattedName(String serviceName) {
        return new ServiceId(serviceName, UUID.randomUUID().toString()).getFormattedName();
    }

}