package org.artifactory.concurrent;

import org.artifactory.common.ConstantValues;

import java.util.concurrent.TimeUnit;

import static org.artifactory.common.ConstantValues.federatedRepoFullSyncLockTimeMin;
import static org.artifactory.common.ConstantValues.haHeartbeatStaleIntervalSecs;
import static org.artifactory.common.ConstantValues.lockLeasTime;

/**
 * Supports dynamic lock expiry & orphan lock criteria for non standard locks
 *
 * @author Uriah Levy
 * @author Mor Merhav
 */
public enum NonStandardLockCategory {

    CLUSTER_SINGLETON("cluster_singleton", ConstantValues.haOrphanClusterSingletonLockPeriod.getInt()),
    REPLICATION("replications", lockLeasTime.getInt() * 24,
            (int) TimeUnit.SECONDS.toMinutes(haHeartbeatStaleIntervalSecs.getLong() * 4L)),
    FORCE_BINARY_SHARDING_OPTIMIZATION("force_optimization", (int) TimeUnit.DAYS.toMinutes(1),
            (int) TimeUnit.DAYS.toMinutes(1)),
    FED_REPO_FULL_SYNC("fed_repo_full_sync", federatedRepoFullSyncLockTimeMin.getInt(),
            (int) TimeUnit.SECONDS.toMinutes(haHeartbeatStaleIntervalSecs.getLong() * 20L));


    String name;
    int lockLeaseTimeMinutes;
    int orphanLockGraceTimeMinutes;

    NonStandardLockCategory(String name, int lockLeaseTimeMinutes, int orphanLockGraceTimeMinutes) {
        this.name = name;
        this.lockLeaseTimeMinutes = lockLeaseTimeMinutes;
        this.orphanLockGraceTimeMinutes = orphanLockGraceTimeMinutes;
    }

    NonStandardLockCategory(String name, int orphanLockGraceTimeMinutes) {
        this(name, -1, orphanLockGraceTimeMinutes);
    }

    public boolean isExpirable() {
        return lockLeaseTimeMinutes != -1;
    }

    public int getLockLeaseTimeMinutes() {
        return lockLeaseTimeMinutes;
    }

    public int getOrphanLockGraceTimeMinutes() {
        return orphanLockGraceTimeMinutes;
    }

    public String getName() {
        return name;
    }

}
