package org.artifactory.descriptor.repo;

import org.testng.Assert;
import org.testng.annotations.Test;

/**
 * @author Volodymyr Shulga
 */
public class HttpRepoDescriptorTest {

    @Test
    public void parseCredentialsFromUrl() {
        HttpRepoDescriptor repoDescriptor = new HttpRepoDescriptor();

        Assert.assertNull(repoDescriptor.extractCredentialsFromUrl("http://localhost:8080/"));

        final String[] credentialsForHttp = repoDescriptor.extractCredentialsFromUrl(
                "https://admin:Password1@localhost:8080/conan-smart-remote"
        );
        Assert.assertNotNull(credentialsForHttp);
        Assert.assertEquals(credentialsForHttp[0], "admin");
        Assert.assertEquals(credentialsForHttp[1], "Password1");

        final String[] credentialsForHttps = repoDescriptor.extractCredentialsFromUrl(
                "http://admin:Password1@localhost:8080/conan-smart-remote"
        );
        Assert.assertNotNull(credentialsForHttps);
        Assert.assertEquals(credentialsForHttps[0], "admin");
        Assert.assertEquals(credentialsForHttps[1], "Password1");

        final String[] credentialsForEncodedAtSymbol = repoDescriptor.extractCredentialsFromUrl(
                "https://localhost:8080/%40conan-smart-remote"
        );
        Assert.assertNull(credentialsForEncodedAtSymbol);

        final String[] credentialsForNonEncodedAtSymbol = repoDescriptor.extractCredentialsFromUrl(
                "https://localhost:8080/@conan-smart-remote"
        );
        Assert.assertNull(credentialsForNonEncodedAtSymbol);

        final String[] nullCredentials = repoDescriptor.extractCredentialsFromUrl(
                "https://null:null@localhost:8080"
        );
        Assert.assertNotNull(nullCredentials);
        Assert.assertEquals(nullCredentials[0], "null");
        Assert.assertEquals(nullCredentials[1], "null");

        final String[] emptyCredentialsWithDots = repoDescriptor.extractCredentialsFromUrl(
                "https://:@localhost:8080"
        );
        Assert.assertNull(emptyCredentialsWithDots);

        final String[] emptyCredentials = repoDescriptor.extractCredentialsFromUrl(
                "https://aaa@localhost:8080"
        );
        Assert.assertNull(emptyCredentials);

        final String[] nullURL = repoDescriptor.extractCredentialsFromUrl(null);
        Assert.assertNull(nullURL);
    }

}