package org.artifactory.ui.utils;

import com.google.common.collect.Maps;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.ui.rest.model.onboarding.QuickRepositoryModel;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import java.util.Map;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertTrue;

/**
 * @author Alexei Vainshtein
 */
public class DefaultRepoCreatorTest {

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testIsDefaultDeploymentRepoValid() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        MutableCentralConfigDescriptor descriptor = mock(MutableCentralConfigDescriptor.class);
        Map<String, LocalRepoDescriptor> reposMap = Maps.newLinkedHashMap();
        String repoName = "repo";
        reposMap.put(repoName, new FederatedRepoDescriptor());
        when(descriptor.getLocalAndFederatedRepositoriesMap()).thenReturn(reposMap);
        CentralConfigService service = mock(CentralConfigService.class);
        when(service.getMutableDescriptor()).thenReturn(descriptor);
        when(context.getCentralConfig()).thenReturn(service);
        DefaultRepoCreator repoCreator = new DefaultRepoCreator();
        QuickRepositoryModel model = new QuickRepositoryModel();
        model.setDefaultDeploymentRepo(repoName);
        assertTrue(repoCreator.isDefaultDeploymentRepoValid(model));
    }
}